\name{adjamat}
\alias{adjamat}
\alias{adjamat.mathgraph}
\alias{is.adjamat}

\title{ Adjacency Matrix of a Mathematical Graph }
\description{
  Returns an object of class \code{"adjamat"} which is the adjacency 
  matrix of a numbered graph.
}
\usage{
adjamat(x, ...)
adjamat.mathgraph(x, general=FALSE, ...)

is.adjamat(x)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{x}{ an object of class \code{"mathgraph"}. }
  \item{general}{ logical flag, if \code{TRUE}, then multiple edges or 
	arcs between the same nodes are counted; otherwise, there is 
	a \code{1} no matter how many edges or arcs there are.
  }
  \item{\dots}{ other arguments for generic function. }
}
\details{
  \code{adjamat} is a generic function with a method for 
  class \code{"mathgraph"}.
}
\value{
  An object of class \code{"adjamat"} which is a square matrix with as many 
  rows and columns as there are nodes in the numbered graph.\cr
  The \code{i},\code{j} element is an indicator of an arc from 
  node \code{i} to node \code{j}.
  An undirected edge between nodes \code{i} and \code{j} contributes 
  a \code{1} to both the \code{i},\code{j} element and 
  the \code{j},\code{i} element.

  \code{is.adjamat} is the membership function for this class.
}
\references{ Chachra, V., Ghare, P. M. and Moore, J. M. (1979). 
	Applications of Graph Theory Algorithms.
	Elvesier North Holland, New York.

	Harary, Frank (1969). Graph Theory, p. 158.
	Addison Wesley.
}
\author{ Nick Efthymiou }
\note{ S Poetry, Patrick J. Burns, \url{http://www.burns-stat.com/pages/spoetry.html} }

\section{BUGS }{The \code{general} argument to \code{adjamat.mathgraph} is not 
functional.} 

\seealso{ \code{\link{mathgraph}}, \code{\link{incidmat}}, 
\code{\link{getpath.adjamat}} }

\examples{
adjamat(mathgraph(~ 1:3 * 3:5, dir=TRUE))
}
\keyword{ math }

\eof
\name{alldirected}
\alias{alldirected}
\alias{alldirected.mathgraph}
\alias{alldirected.default}

\title{ Transform to Directed Graph }
\description{
  Returns a \code{"mathgraph"} object which has all edges directed.
}
\usage{
alldirected(x, ...)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{x}{ an object representing a mathematical graph. }
  \item{\dots}{ other arguments for derived functions. }
}
\details{
  The default method merely creates an error.
}
\value{
  A \code{"mathgraph"} object with any undirected edges in the 
  input split into two arcs.
}
\references{ S Poetry, Patrick J. Burns, Section 13.3, p. 305 }
\author{ Nick Efthymiou }
%\note{ ~~further notes~~ }

\seealso{ \code{\link{mathgraph}} }

\examples{
my.graph <- mathgraph(~ 1:3 / 2:4) # undirected graph with 3 edges
alldirected(my.graph) # directed graph with 6 arcs

}
\keyword{ math }

\eof
\name{build.mathgraph}
\alias{build.mathgraph}

\title{ Internal Function for Mathematical Graphs }
\description{
  This is an internal function.
  It is not meant for direct use.
}
\usage{
build.mathgraph(formula, data)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{formula}{ see \code{mathgraph} }
  \item{data}{ see \code{mathgraph} }
}
%\details{
%  ~~ If necessary, more details than the __description__  above ~~
%}
%\value{
%}
\references{ S Poetry, Patrick J. Burns }
\author{ Nick Efthymiou }
\note{ Temporary objects may be left over when I() expressions are used.
	The following command cleans them: \cr
	rm(list=ls(pattern="Build.mathgraphI"))
}

\seealso{ \code{\link{mathgraph}} }

%\examples{
%}
\keyword{ internal }

\eof
\name{commontail}
\alias{commontail}

\title{ Common Strings in Tail }
\description{
  Takes a list of character vectors and returns the longest vector
  of strings that is common to the ends of all of the components in the list.
}
\usage{
commontail(x)
}

\arguments{
  \item{x}{ list of character vectors. }
}
\details{
  This is useful to get the class that is common to a number of objects.
}
\value{
  a character vector containing the common elements of the tails
  of all the components in \code{x}.\cr
  The result is \code{NULL} if there are no common elements.
}
\references{ S Poetry, Patrick J. Burns }
\author{ Nick Efthymiou }
%\note{ ~~further notes~~ }

\seealso{ \code{\link{inherits}}, \code{\link{intersect}} }

\examples{
commontail(list(c("subA", "cls1"), c("subB", "subA", "cls1")))
commontail(list(c("subA", "cls2"), c("subB", "subA", "cls1")))
}
\keyword{ programming }
\keyword{ methods }

\eof
\name{.find.I.of}
\alias{.find.I.of}

\title{ Find limits of I() in a String }
\description{
  Returns either \code{NULL} or a two-column matrix where each row gives the
  first and last character of a call to \code{I}.
}
\usage{
.find.I.of(string, nesting.ok = FALSE)
}

\arguments{
  \item{string}{ a single character string. }
  \item{nesting.ok}{ logical value: if \code{TRUE}, then all occurrences 
	are given.
	If \code{FALSE}, then calls to I within other calls to it are ignored.}
}
%\details{
%  ~~ If necessary, more details than the __description__  above ~~
%}
\value{
  a numeric matrix with two columns, or \code{NULL}.
  Each row represents one call to I.\cr
  The first column is the number of the character within the string that
  starts the call (i.e., the "I").\cr
  The second column is the number of the character within the string that
  ends the call (the ")").
}
\references{ S Poetry, Patrick J. Burns, Section 11.2, p. 280 }
\author{ Nick Efthymiou }
\note{ This is used by mathgraph functions, and is not meant for direct use. }
\section{BUGS}{Parentheses inside quotes in the call to I will confuse it.

One or more spaces after the "I" will cause the call to be unrecognized.
However, if the string is parsed and then deparsed, the space will go away.
}

\seealso{ \code{\link{build.mathgraph}} }

\examples{
.find.I.of("~ x + I(x^2)")
.find.I.of("~ x + I(x^2 + I(y^3))")
\dontrun{
.find.I.of("~ x + I(x^2 + I(y^3))", TRUE)
}
}
\keyword{ programming }
\keyword{ internal }

\eof
\name{getpath}
\alias{getpath}
\alias{getpath.mathgraph}
\alias{getpath.incidmat}
\alias{getpath.adjamat}
\alias{getpath.default}

\title{ Find a Path in a Mathematical Graph }
\description{
  Returns a path, if it exists, from the \code{start} to the \code{end}.
}
\usage{
getpath(x, start, end, ...)
getpath.mathgraph(x, start, end, ...)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{x}{ an object representing a mathematical graph. }
  \item{start}{ character string or integer giving the starting node. }
  \item{end}{ character string or integer giving the ending node. }
  \item{\dots}{ generic arguments. }
}
\details{
  \code{getpath} is a generic function with methods for
  \code{"mathgraph"}, \code{"incidmat"} and \code{"adjamat"}.
  The default method converts \code{x} to class \code{"incidmat"}.

  \code{getpath.adjamat} is an implementation of algorithm 2.2 in Chachra, 
  Ghare and Moore (1979) and \code{getpath.incidmat} is an implementation 
  of their algorithm 2.3.

  The distinction between non-existent paths and paths of length zero
  may be useful in some situations.
}
\value{
  When at least one path exists, a \code{"mathgraph"} object containing 
  the edges within the first path found; 
  this may be an empty mathgraph if \code{start} and \code{end} are equal.\cr
  When no path exists, returns \code{NULL}. 
}
\references{ Chachra, V., Ghare, P. M. and Moore, J. M. (1979). 
	Applications of Graph Theory Algorithms.
	Elvesier North Holland, New York.
}
\author{ Nick Efthymiou }
\note{ S Poetry, Patrick J. Burns, Section 13.3 }

\seealso{ \code{\link{mathgraph}}, \code{\link{incidmat}}, 
\code{\link{adjamat}} }

\examples{
getpath(mathgraph(~ 1:3 / 3:5), 1, 5) # returns a path
getpath(mathgraph(~ 1:3 / 3:5), 1, 4) # no path, returns NULL
getpath(mathgraph(~ 1:3 / 3:5), 1, 1) # returns mathgraph()

}
\keyword{ math }

\eof
\name{incidmat}
\alias{incidmat}
\alias{incidmat.mathgraph}
\alias{is.incidmat}

\title{ Incidence Matrix for a Mathematical Graph }
\description{
  Returns an object of class \code{"incidmat"} which is a matrix indicating
  the start and end node for each edge in the graph.
}
\usage{
incidmat(x, ...)
incidmat.mathgraph(x, expand=TRUE, general=FALSE, ...)

is.incidmat(x)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{x}{ object representing a mathematical graph. }
  \item{expand}{ logical flag: if \code{TRUE}, then undirected edges are 
	represented by two columns in the output.\cr
	If \code{FALSE}, then both non-zero elements of an undirected edge 
	are positive.
  }
  \item{general}{ logical flag: if \code{TRUE}, then there is a non-zero 
	entry in a column representing a loop.
  }
  \item{\dots}{ generic arguments. }
}
\details{
  The \code{incidmat} function is generic, with a method for 
  class \code{"mathgraph"}.

  \code{is.incidmat} is the membership function for this class.
}
\value{
  an object of class \code{"incidmat"} which is a matrix with rows representing
  nodes and columns representing edges.\cr
  Generally speaking, there is a \code{1} in the location where an edge begins
  and a \code{-1} in the location where it ends.\cr
  Frank Harary defines incidence matrix with expand = \code{FALSE}.
}
\references{ Chachra, V., Ghare, P. M. and Moore, J. M. (1979). 
Applications of Graph Theory Algorithms.
Elvesier North Holland, New York.

Harary, Frank (1969). Graph Theory, p. 160.
Addison Wesley.

}
\author{ Nick Efthymiou }
\note{ From S Poetry, Patrick J. Burns, Section 13.3, p. 312 
	\url{http://www.burns-stat.com/pages/spoetry.html}
}

\seealso{ \code{\link{adjamat}}, \code{\link{mathgraph}},
\code{\link{getpath.incidmat}} }

\examples{
incidmat(mathgraph(~ 1:3 / 3:5, dir=TRUE))
incidmat(mathgraph(~ 1:3 / 3:5, dir=FALSE))
incidmat(mathgraph(~ 1:3 / 3:5, dir=FALSE), expand=FALSE)

}
\keyword{ math }

\eof
\name{justify}
\alias{justify}

\title{ Justify Elements of a Vector }
\description{
  Returns a vector like the input, but each string may have added blank
  spaces at the start and/or end.
}
\usage{
justify(x, type = "r")
}

\arguments{
  \item{x}{ a character vector. }
  \item{type}{ a string giving the type of justification.\cr
	This may be an abbreviation of one of "right", "left", "center".
  }
}
%\details{
%  ~~ If necessary, more details than the __description__  above ~~
%}
\value{
  a character vector like x, except all elements have the same number
  of characters, and the text is lined up along one edge, or centered.
}
\references{ S Poetry, Patrick J. Burns }
\author{ Nick Efthymiou }
%\note{ ~~further notes~~ }

\seealso{ \code{\link{format}}, \code{\link{substring}}, 
\code{\link{paste}} }

\examples{
data(freeny)
as.matrix(justify(dimnames(freeny.x)[[2]], "r"))
as.matrix(justify(dimnames(freeny.x)[[2]], "l"))
as.matrix(justify(dimnames(freeny.x)[[2]], "c"))
}
\keyword{ character }

\eof
\name{mathgraph}
\alias{mathgraph}
\alias{[.mathgraph}
\alias{[<-.mathgraph}
\alias{length.mathgraph}
\alias{c.mathgraph}
\alias{is.mathgraph}

\title{ Create Mathematical Graph }
\description{
  Create an object of class \code{"mathgraph"} which represents a 
  mathematical graph.
}
\usage{
mathgraph(formula, directed = FALSE, data = sys.parent())
length.mathgraph(x)
c.mathgraph(...)

is.mathgraph(x)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{formula}{ a formula containing just the right-side.\cr
	Special operators in the formula are \code{+} which separates 
	terms, \code{/} which puts an edge between corresponding 
	elements of the two vectors on which it is operating, 
	and \code{*} which puts an edge between every pair of elements
	in the two vectors on which it is operating.
  }
  \item{directed}{ logical flag: if \code{TRUE}, then all edges that 
	are created are directed, otherwise they are undirected.
  }
  \item{data}{ the frame in which to find objects referenced in the formula.
	This can be either the number of a memory frame, or a list or 
	data frame containing the data.
  }
  \item{\dots}{ objects to be concatenated. }
  \item{x}{ object of class \code{"mathgraph"}. }
}
\details{
  Mathematical graphs consist of a set of nodes (vertices) and edges.
  Edges go between two nodes.
  An edge that is directed is often called an arc.

  Terms in the formula (delimited by \code{+})
  may be either calls to \code{*} or \code{/}, or objects that
  are already of class \code{"mathgraph"}.

  Two other representations of graphs are adjacency matrices and incidence
  matrices.
  The functions to convert \code{"mathgraph"} objects to these 
  are \code{adjamat} and \code{incidmat}, respectively.
  Most algorithms for mathematical graphs are in terms of incidence matrices
  or adjacency matrices.

  The generic functions that have a method for class \code{"mathgraph"} include:
  \code{[}, \code{c}, \code{length}, \code{names}, \code{plot}, \code{print}, 
  \code{unique}.

  \code{is.mathgraph} is the membership function for this class.

}
\value{
  an object of class \code{mathgraph} which is a two-column matrix of nodes
  along with an additional attribute called \code{"directed"} which is a 
  logical vector stating whether or not each edge is directed.\cr
  An edge (row of the matrix) that is directed goes from the node in the first
  column to the node in the second column.
}
\references{ Chachra, V., Ghare, P. M. and Moore, J. M. (1979).
	Applications of Graph Theory Algorithms.
	Elvesier North Holland, New York.
}
\author{ Nick Efthymiou }
\note{ S Poetry, Patrick J. Burns, \url{http://www.burns-stat.com/pages/spoetry.html} 
}

\seealso{ \code{\link{adjamat}}, \code{\link{incidmat}}, \code{\link{getpath}} }

\examples{

mathgraph(~ 1:3 / 2:4) # graph with 3 edges
mathgraph(~ 1:3 * 2:4) # graph with 9 edges

mathgraph(~ 1:3 / 2:4, dir=TRUE) # directed graph with 3 edges

# graph with some edges directed, some not
c(mathgraph(~ 1:3 * 2:4), mathgraph(~ c(3,1) / c(2,4), dir=TRUE))

}
\keyword{ math }

\eof
\name{names.mathgraph}
\alias{names.mathgraph}
\alias{names<-.mathgraph}

\title{ Edge Names in a Mathematical Graph }
\description{
  Sets or returns the names (corresponding to the edges) of a mathematical
  graph represented by a \code{mathgraph} object.
}
\usage{
\method{names}{mathgraph}(x)
\method{names}{mathgraph}(x) <- value
}

\arguments{
  \item{x}{ an object inheriting from \code{mathgraph}. }
  \item{value}{ a value to be assigned to the names of the \code{mathgraph}. }
}
\details{
  In the assignment form, the names are created or changed.
}
\value{
  Character vector of the names.
}
\references{ S Poetry, Patrick J. Burns, Section 13.3 Mathematical Graphs, 
	\url{http://www.burns-stat.com/pages/spoetry.html}
}
\author{ Nick Efthymiou }
%\note{ ~~further notes~~ }

\seealso{ \code{\link{mathgraph}} }

\examples{

jjm <- mathgraph(~ 1:3 * 2:4)
jjm
names(jjm) <- letters[1:length(jjm)]
jjm
names(jjm)

}
\keyword{ math }

\eof
\name{plot.mathgraph}
\alias{plot.mathgraph}

\title{ Plot a Mathematical Graph }
\description{
  Very crude plotting method for \code{mathgraph} class.
}
\usage{
\method{plot}{mathgraph}(x, ...)
}

\arguments{
  \item{x}{ an object that inherits from \code{mathgraph}. }
  \item{\dots}{ graphics parameters may be given. }
}
\details{
  A representation of the mathematical graph is produced on
  the current graphics device.
}
%\value{
%}
\references{ S Poetry, Patrick J. Burns, Section 13.3 }
\author{ Nick Efthymiou }
%\note{ ~~further notes~~ }

\section{BUGS }{Needs to be smarter, and allow the user some control.}

\seealso{ \code{\link{mathgraph}}, \code{\link{plot}}, \code{\link{par}} }

\examples{
plot(mathgraph(~ 1:3 * 2:4), main="Graph K4")
}
\keyword{ hplot }
\keyword{ math }

\eof
\name{print.mathgraph}
\alias{print.mathgraph}

\title{ Print a Mathematical Graph }
\description{
  Prints a representation of the graph.
}
\usage{
\method{print}{mathgraph}(x, prefix.node = if (is.character(xu)) "" else "node", ...)
}

\arguments{
  \item{x}{ an object inheriting from mathgraph which represents a 
	mathematical graph. 
  }
  \item{prefix.node}{ a string to put in front of each node named. 
	The default is an empty string if the nodes are character and the
	string "node" if they are not.
  }
  \item{\dots}{ other arguments to print may be given, but are not used. }
}
\details{
  The object is printed.\cr
  A `--' between nodes means an undirected edge, while a single
  arrow means a directed edge.
}
\value{
  the input x is returned invisibly.
}
\references{ S Poetry, Patrick J. Burns, Section 13.3, Mathematical Graphs }
\author{ Nick Efthymiou }
\note{ The format is consistent with the dot graph language. }

\seealso{ \code{\link{mathgraph}}, \code{\link{names.mathgraph}} }

\examples{
mathgraph(~ 1:3 / 2:4)
mathgraph(~ 1:3 / 2:4, dir=TRUE)
jjm <- mathgraph(~ letters[1:3] * letters[2:4])
jjm
names(jjm) <- LETTERS[1:9]
jjm

}
\keyword{ math }

\eof
\name{sort.mathgraph}
\alias{sort.mathgraph}

\title{ Sort a Mathematical Graph }
\description{
  Sorts nodes within undirected edges and/or edges by nodes.
}
\usage{
sort.mathgraph(x, nodes = TRUE, edges = TRUE)
}

\arguments{
  \item{x}{ an object that inherits from \code{mathgraph}. }
  \item{nodes}{ logical value; if \code{TRUE}, then the nodes within 
	undirected edges are sorted. 
  }
  \item{edges}{ logical value; if \code{TRUE}, then the edges are sorted by 
	the first node with ties broken by the second node.
  }
}
%\details{
%  ~~ If necessary, more details than the __description__  above ~~
%}
\value{
  an object that represents the same graph as the input, but with
  some rearrangement.
}
\references{ S Poetry, Patrick J. Burns, Section 13.3; 
	\url{http://www.burns-stat.com/pages/spoetry.html}
}
\author{ Nick Efthymiou }
% \note{ ~~further notes~~ }

\seealso{ \code{\link{mathgraph}} }

\examples{
jjmg <- c(mathgraph(~ 4:2 * 1:3), mathgraph(~ 3:5 / 1:3))
sort.mathgraph(jjmg)
sort.mathgraph(jjmg, node=FALSE)
sort.mathgraph(jjmg, edge=FALSE)

}
\keyword{ math }

\eof
\name{stable.apply}
\alias{stable.apply}

\title{ Apply with Stable Dimensions }
\description{
  Does the same thing as \code{apply} except that when the function returns a
  vector, the dimensions are put back the way they started.
}
\usage{
stable.apply(X, MARGIN, FUN, ...)
}

\arguments{
  \item{X}{ same as in \code{apply}. }
  \item{MARGIN}{ same as in \code{apply}. }
  \item{FUN}{ same as in \code{apply}. }
  \item{\dots}{ same as in \code{apply}. }
}
%\details{
%  ~~ If necessary, more details than the __description__  above ~~
%}
\value{
  When \code{FUN} returns a scalar or when \code{MARGIN} has a length 
  that is not one less than the number of dimensions in \code{X}, then 
  the same as \code{apply}.

  Otherwise, an array similar to the result of \code{apply}, but with the 
  dimensions permuted to correspond to the dimensions of \code{X}.
}
\references{ S Poetry, Patrick J. Burns, 
	\url{http://www.burns-stat.com/pages/spoetry.html}
}
\author{ Nick Efthymiou }
%\note{ ~~further notes~~ }

\seealso{ \code{\link{apply}}, \code{\link{aperm}} }

\examples{
data(freeny)
stable.apply(freeny.x, 1, sort)
# compare to:
apply(freeny.x, 1, sort)

}
\keyword{ array }

\eof
\name{unique.mathgraph}
\alias{unique.mathgraph}

\title{ Unique Edges of a Mathematical Graph }
\description{
  Returns a \code{mathgraph} object that may have fewer edges than the input.
}
\usage{
\method{unique}{mathgraph}(x, incomparables = FALSE, ...)
}

\arguments{
  \item{x}{ an object that inherits from \code{mathgraph}. }
  \item{incomparables}{ a vector of values that cannot be compared. 
	The only possible value is \code{FALSE}, meaning that all
	values can be compared.
  }
  \item{\dots}{ other arguments for generic function. }
}
%\details{
%  ~~ If necessary, more details than the __description__  above ~~
%}
\value{
  An object that is the same class as the input \code{x}, but redundant edges
  are removed.
}
\references{ S Poetry, Patrick J. Burns, Section 13.3 }
\author{ Nick Efthymiou }
%\note{ ~~further notes~~ }

\seealso{ \code{\link{mathgraph}} }

\examples{
unique( c(mathgraph(~ 1:2*2:3), mathgraph(~ 1/3)) )
}
\keyword{ math }
\keyword{ methods }

\eof
